#!/usr/bin/env python
import logging
import os
import sys
from os.path import abspath, dirname

import django
from django.conf import settings
from django.test.runner import DiscoverRunner
from edc_test_utils import DefaultTestSettings

app_name = "edc_model_admin"
base_dir = dirname(abspath(__file__))

DEFAULT_SETTINGS = DefaultTestSettings(
    calling_file=__file__,
    template_dirs=[os.path.join(base_dir, app_name, "tests", "templates")],
    BASE_DIR=base_dir,
    APP_NAME=app_name,
    ETC_DIR=os.path.join(base_dir, app_name, "tests", "etc"),
    SUBJECT_VISIT_MODEL="edc_model_admin.subjectvisit",
    INSTALLED_APPS=[
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.staticfiles",
        "django.contrib.sites",
        "django_extensions",
        "django_crypto_fields.apps.AppConfig",
        "django_revision.apps.AppConfig",
        "edc_appointment.apps.AppConfig",
        "edc_consent.apps.AppConfig",
        "edc_crf.apps.AppConfig",
        "edc_dashboard.apps.AppConfig",
        "edc_device.apps.AppConfig",
        "edc_identifier.apps.AppConfig",
        "edc_facility.apps.AppConfig",
        "edc_metadata.apps.AppConfig",
        "edc_lab.apps.AppConfig",
        "edc_navbar.apps.AppConfig",
        "edc_offstudy.apps.AppConfig",
        "edc_protocol.apps.AppConfig",
        "edc_randomization.apps.AppConfig",
        "edc_reference.apps.AppConfig",
        "edc_registration.apps.AppConfig",
        "edc_sites.apps.AppConfig",
        "edc_subject_dashboard.apps.AppConfig",
        "edc_timepoint.apps.AppConfig",
        "edc_visit_schedule.apps.AppConfig",
        "edc_visit_tracking.apps.AppConfig",
        "edc_model_admin.apps.AppConfig",
    ],
    DASHBOARD_BASE_TEMPLATES={
        "dashboard_template": os.path.join(
            base_dir, "edc_model_admin", "tests", "templates", "dashboard.html"
        ),
        "dashboard2_template": os.path.join(
            base_dir, "edc_model_admin", "tests", "templates", "dashboard2.html"
        ),
    },
    use_test_urls=True,
    add_dashboard_middleware=True,
).settings


def main():
    if not settings.configured:
        settings.configure(**DEFAULT_SETTINGS)
    django.setup()
    tags = [t.split("=")[1] for t in sys.argv if t.startswith("--tag")]
    failures = DiscoverRunner(failfast=False, tags=tags).run_tests([f"{app_name}.tests"])
    sys.exit(failures)


if __name__ == "__main__":
    logging.basicConfig()
    main()



from django.apps import apps as django_apps
from django.conf import settings
from django.contrib import admin
from django.contrib.admin import AdminSite as DjangoAdminSite
from django.contrib.sites.shortcuts import get_current_site
from edc_protocol import Protocol

admin.site.enable_nav_sidebar = False


class EdcAdminSite(DjangoAdminSite):

    """
    Add to your project urls.py
        path("edc_action_item/", edc_action_item.urls),

    -OR-
    To include this in the administration section set
    `AppConfig.include_in_administration_section = True`
    in your apps.py. (See also View `edc_dashboard.administration.py`).

    If set to `include_in_administration_section=True`, add a local `urls.py`

        from django.urls.conf import path
        from django.views.generic import RedirectView

        app_name = "edc_action_item"

        urlpatterns = [
            path("", RedirectView.as_view(url="admin/"), name="home_url"),
        ]

    and then add to your project urls.py

        path("edc_action_item/admin/", edc_action_item_admin.urls),
        path("edc_action_item/", include("edc_action_item.urls")),

    """

    app_index_template = "edc_model_admin/admin/app_index.html"
    login_template = f"edc_auth/bootstrap{settings.EDC_BOOTSTRAP}/login.html"
    logout_template = f"edc_auth/bootstrap{settings.EDC_BOOTSTRAP}/login.html"
    enable_nav_sidebar = False  # DJ 3.1
    final_catch_all_view = True  # DJ 3.2
    site_url = "/administration/"

    def __init__(self, name="admin", app_label=None, keep_delete_action=None):
        self.app_label = app_label
        super().__init__(name)
        if not keep_delete_action:
            del self._actions["delete_selected"]

    def each_context(self, request):
        context = super().each_context(request)
        context.update(global_site=get_current_site(request))
        context.update(
            site_title=self.get_edc_site_title(request),
            site_header=self.get_edc_site_header(request),
            index_title=self.index_title,
        )
        return context

    def get_edc_site_title(self, request):
        verbose_name = django_apps.get_app_config(self.app_label).verbose_name
        return verbose_name.replace(
            Protocol().project_name,
            f"{Protocol().project_name} @ {get_current_site(request).name.title()} ",
        )

    def get_edc_site_header(self, request):
        return self.get_edc_site_title(request)



from django.urls import reverse

from .base_model_admin_redirect_mixin import BaseModelAdminRedirectMixin


class ModelAdminModelRedirectMixin(BaseModelAdminRedirectMixin):

    """Redirect to another model's changelist on add, change or delete."""

    redirect_app_label = None
    redirect_model_name = None
    redirect_search_field = None
    redirect_namespace = "admin"

    def search_value(self, obj):
        def objattr(inst):
            my_inst = inst
            if self.redirect_search_field:
                for name in self.redirect_search_field.split("__"):
                    my_inst = getattr(my_inst, name)
            return my_inst

        try:
            value = objattr(obj)
        except TypeError:
            value = None
        return value

    def redirect_url(self, request, obj, post_url_continue=None, namespace=None):
        namespace = namespace or self.redirect_namespace
        return "{}?q={}".format(
            reverse(
                "{namespace}:{app_label}_{model_name}_changelist".format(
                    namespace=namespace,
                    app_label=self.redirect_app_label,
                    model_name=self.redirect_model_name,
                )
            ),
            self.search_value(obj) or "",
        )

    def redirect_url_on_delete(self, request, obj_display, obj_id, namespace=None):
        namespace = namespace or self.redirect_namespace
        return reverse(
            "{namespace}:{app_label}_{model_name}_changelist".format(
                namespace=namespace,
                app_label=self.redirect_app_label,
                model_name=self.redirect_model_name,
            )
        )



from edc_protocol import Protocol


class ModelAdminInstitutionMixin:
    """Adds institution attrs to the ModelAdmin context."""

    @staticmethod
    def get_institution_extra_context(extra_context):
        protocol = Protocol()
        extra_context.update(
            {
                "institution": protocol.institution,
                "copyright": protocol.copyright,
                "license": protocol.license or "",
                "disclaimer": protocol.disclaimer,
            }
        )
        return extra_context

    def add_view(self, request, form_url="", extra_context=None):
        extra_context = extra_context or {}
        extra_context = self.get_institution_extra_context(extra_context)
        return super().add_view(request, form_url=form_url, extra_context=extra_context)

    def change_view(self, request, object_id, form_url="", extra_context=None):
        extra_context = extra_context or {}
        extra_context = self.get_institution_extra_context(extra_context)
        return super().change_view(
            request, object_id, form_url=form_url, extra_context=extra_context
        )



class LimitedAdminInlineMixin:
    """Limit choices on a foreignkey field in an inline to a value
    on the parent model.

    From getresults_csv. For example, model CsvDictionary is an
    inline to CsvFormat. The inline has a foreignkey to model
    CsvFields. By using the mixin, csv_field is filtered by the
    value of csv_format instead of showing everything in CsvFields.

        class CsvDictionaryInline(LimitedAdminInlineMixin,
                                  admin.TabularInline):
            model = CsvDictionary
            form = CsvDictionaryForm
            extra = 0

            def get_filters(self, obj):
                if obj:
                    return (('csv_field', dict(csv_format=obj.id)),)
                else:
                    return ()

        class CsvFormatAdmin(admin.ModelAdmin):
            inlines = [CsvDictionaryInline]
        admin_site.register(CsvFormat, CsvFormatAdmin)
    """

    @staticmethod
    def limit_inline_choices(formset, field, empty=False, **filters):
        assert field in formset.form.base_fields
        qs = formset.form.base_fields[field].queryset
        if empty:
            formset.form.base_fields[field].queryset = qs.none()
        else:
            qs = qs.filter(**filters)
            formset.form.base_fields[field].queryset = qs

    def get_formset(self, request, obj=None, **kwargs):
        formset = super(LimitedAdminInlineMixin, self).get_formset(request, obj, **kwargs)

        for (field, filters) in self.get_filters(obj):
            if obj:
                self.limit_inline_choices(formset, field, **filters)
            else:
                self.limit_inline_choices(formset, field, empty=True)

        return formset

    def get_filters(self, obj):
        return getattr(self, "filters", ())



from edc_reference import ReferenceModelConfig, site_reference_configs


def register_to_site_reference_configs():
    site_reference_configs.registry = {}

    reference = ReferenceModelConfig(name="edc_model_admin.CrfOne", fields=["f1"])
    site_reference_configs.register(reference)

    reference = ReferenceModelConfig(name="edc_model_admin.CrfTwo", fields=["f1"])
    site_reference_configs.register(reference)

    reference = ReferenceModelConfig(name="edc_model_admin.CrfThree", fields=["f1"])
    site_reference_configs.register(reference)

    reference = ReferenceModelConfig(name="edc_model_admin.CrfFour", fields=["f1"])
    site_reference_configs.register(reference)

    reference = ReferenceModelConfig(name="edc_model_admin.CrfFive", fields=["f1"])
    site_reference_configs.register(reference)

    reference = ReferenceModelConfig(name="edc_model_admin.CrfSix", fields=["f1"])
    site_reference_configs.register(reference)

    reference = ReferenceModelConfig(name="edc_model_admin.CrfSeven", fields=["f1"])
    site_reference_configs.register(reference)

    reference = ReferenceModelConfig(name="edc_model_admin.CrfMissingManager", fields=["f1"])
    site_reference_configs.register(reference)



from django.urls import reverse

from .model_admin_changelist_button_mixin import ModelAdminChangelistButtonMixin


class ModelAdminChangelistModelButtonMixin(ModelAdminChangelistButtonMixin):

    """Use a button as a list_display field with a link to add,
    change or changelist.
    """

    def changelist_model_button(
        self,
        app_label,
        model_name,
        reverse_args=None,
        namespace=None,
        change_label=None,
        add_label=None,
        add_querystring=None,
        disabled=None,
        title=None,
    ):
        if disabled:
            changelist_model_button = self.disabled_button(add_label or change_label)
        else:
            app_label = app_label
            model_name = model_name
            if reverse_args:
                changelist_model_button = self.change_model_button(
                    app_label,
                    model_name,
                    reverse_args,
                    namespace=namespace,
                    label=change_label,
                    title=title,
                )
            else:
                changelist_model_button = self.add_model_button(
                    app_label,
                    model_name,
                    label=add_label,
                    querystring=add_querystring,
                    namespace=namespace,
                    title=title,
                )
        return changelist_model_button

    def change_model_button(
        self,
        app_label,
        model_name,
        reverse_args,
        label=None,
        namespace=None,
        title=None,
    ):
        label = label or "change"
        namespace = namespace or "admin"
        url = reverse(f"{namespace}:{app_label}_{model_name}_change", args=reverse_args)
        return self.button_template(label, url=url, title=title)

    def add_model_button(
        self,
        app_label,
        model_name,
        label=None,
        querystring=None,
        namespace=None,
        title=None,
    ):
        label = label or "add"
        namespace = namespace or "admin"
        url = reverse((f"{namespace}:{app_label}_{model_name}_add") + (querystring or ""))
        return self.button_template(label, url=url, title=title)

    def changelist_list_button(
        self,
        app_label,
        model_name,
        querystring_value=None,
        label=None,
        disabled=None,
        namespace=None,
        title=None,
    ):
        """Return a button that goes to the app changelist filter for
        this model instance.
        """
        label = label or "change"
        namespace = namespace or "admin"
        querystring = ""
        if querystring_value:
            querystring = f"?q={querystring_value}"
        url = reverse((f"{namespace}:{app_label}_{model_name}_changelist") + querystring)
        return self.button_template(label, disabled=disabled, title=title, url=url)

    def disabled_button(self, label):
        return self.button_template(label, disabled="disabled", url="#")



import uuid

from django.db import models
from edc_consent.field_mixins import PersonalFieldsMixin
from edc_consent.field_mixins.identity_fields_mixin import IdentityFieldsMixin
from edc_consent.model_mixins import ConsentModelMixin
from edc_identifier.managers import SubjectIdentifierManager
from edc_identifier.model_mixins import UniqueSubjectIdentifierFieldMixin
from edc_lab.model_mixins import RequisitionModelMixin
from edc_metadata.model_mixins.creates import CreatesMetadataModelMixin
from edc_model.models import BaseUuidModel
from edc_registration.model_mixins import UpdatesOrCreatesRegistrationModelMixin
from edc_sites.models import SiteModelMixin
from edc_visit_schedule.model_mixins import OffScheduleModelMixin, OnScheduleModelMixin
from edc_visit_schedule.model_mixins.subject_on_schedule_model_mixin import (
    SubjectOnScheduleModelMixin,
)
from edc_visit_schedule.model_mixins.visit_schedule_model_mixins import (
    VisitScheduleFieldsModelMixin,
    VisitScheduleMethodsModelMixin,
)
from edc_visit_tracking.model_mixins import VisitModelMixin, VisitTrackingCrfModelMixin


class BasicModel(SiteModelMixin, BaseUuidModel):

    f1 = models.CharField(max_length=10)
    f2 = models.CharField(max_length=10)
    f3 = models.CharField(max_length=10, null=True, blank=False)
    f4 = models.CharField(max_length=10, null=True, blank=False)
    f5 = models.CharField(max_length=10)
    f5_other = models.CharField(max_length=10, null=True)
    subject_identifier = models.CharField(max_length=25, default="12345")


class OnSchedule(OnScheduleModelMixin, BaseUuidModel):

    pass


class OffSchedule(OffScheduleModelMixin, BaseUuidModel):

    pass


class DeathReport(UniqueSubjectIdentifierFieldMixin, SiteModelMixin, BaseUuidModel):

    objects = SubjectIdentifierManager()

    def natural_key(self):
        return (self.subject_identifier,)


class SubjectConsent(
    ConsentModelMixin,
    PersonalFieldsMixin,
    IdentityFieldsMixin,
    UniqueSubjectIdentifierFieldMixin,
    UpdatesOrCreatesRegistrationModelMixin,
    SiteModelMixin,
    SubjectOnScheduleModelMixin,
    VisitScheduleFieldsModelMixin,
    VisitScheduleMethodsModelMixin,
    BaseUuidModel,
):

    objects = SubjectIdentifierManager()

    def natural_key(self):
        return (self.subject_identifier,)


class SubjectVisit(VisitModelMixin, CreatesMetadataModelMixin, SiteModelMixin, BaseUuidModel):
    def update_reference_on_save(self):
        pass


class Requisition(RequisitionModelMixin, BaseUuidModel):
    def update_reference_on_save(self):
        pass


class BaseCrfModel(VisitTrackingCrfModelMixin, SiteModelMixin, models.Model):

    f1 = models.CharField(max_length=50, default=uuid.uuid4)

    class Meta:
        abstract = True


class CrfOne(BaseCrfModel, BaseUuidModel):
    pass


class CrfTwo(BaseCrfModel, BaseUuidModel):

    pass


class CrfThree(BaseCrfModel, BaseUuidModel):

    pass


class CrfFour(BaseCrfModel, BaseUuidModel):

    pass


class CrfFive(BaseCrfModel, BaseUuidModel):

    pass


class CrfSix(BaseCrfModel, BaseUuidModel):

    pass


class CrfSeven(BaseCrfModel, BaseUuidModel):

    pass


class RedirectModel(BaseUuidModel):

    subject_identifier = models.CharField(max_length=25)


class RedirectNextModel(BaseUuidModel):

    subject_identifier = models.CharField(max_length=25)



from django.test import TestCase
from django.test.client import RequestFactory

from ...model_admin_next_url_redirect_mixin import (
    ModelAdminNextUrlRedirectError,
    ModelAdminNextUrlRedirectMixin,
)
from ..models import BasicModel


class TestModelAdmin(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_next_url(self):
        obj = BasicModel()
        request = self.factory.get(
            "/?next=my_url_name,arg1,arg2&agr1=value1&arg2=value2&arg3=value3&arg4=value4"
        )
        mixin = ModelAdminNextUrlRedirectMixin()
        self.assertRaises(ModelAdminNextUrlRedirectError, mixin.redirect_url, request, obj)

    def test_next_url1(self):
        obj = BasicModel()
        request = self.factory.get(
            "/?next=my_url_name,arg1,arg2&arg1=value1&arg2=value2&arg3=value3&arg4=value4"
        )
        mixin = ModelAdminNextUrlRedirectMixin()
        try:
            mixin.redirect_url(request, obj)
        except ModelAdminNextUrlRedirectError as e:
            self.assertIn("my_url_name", str(e))

    def test_next_url4(self):
        obj = BasicModel()
        request = self.factory.get(
            "/?next=my_url_name,arg1,arg2&arg1=value1&arg2=value2&arg3=value3&arg4=value4"
        )
        mixin = ModelAdminNextUrlRedirectMixin()
        try:
            mixin.redirect_url(request, obj)
        except ModelAdminNextUrlRedirectError as e:
            self.assertIn("{'arg1': 'value1', 'arg2': 'value2'}", str(e))



from .model_admin_crf_dashboard_mixin import ModelAdminCrfDashboardMixin
from .model_admin_stacked_inline_mixin import ModelAdminStackedInlineMixin
from .model_admin_subject_dashboard_mixin import ModelAdminSubjectDashboardMixin



class TemplatesModelAdminMixin:
    """Override admin templates.

    Note: If using inlines.

    On the inline admin class specify the position `after` with class
    attribute `insert_after`:

    For example:
        class MyInlineModelAdmin(..):
            ...
            insert_after="<fieldname>"
            ...

    See also: https://linevi.ch/en/django-inline-in-fieldset.html
    """

    show_object_tools = False

    add_form_template = "edc_model_admin/admin/change_form.html"
    change_form_template = "edc_model_admin/admin/change_form.html"
    change_list_template = "edc_model_admin/admin/change_list.html"

    def changelist_view(self, request, extra_context=None):
        extra_context = {} if not extra_context else extra_context
        extra_context.update({"show_object_tools": self.show_object_tools})
        return super().changelist_view(request, extra_context=extra_context)



from django.contrib import messages
from django.http.response import HttpResponseRedirect
from django.urls import reverse
from django.utils.encoding import force_str
from edc_dashboard.url_names import InvalidUrlName, url_names


class ModelAdminRedirectOnDeleteMixin:

    """A mixin to redirect on delete.

    If `post_url_on_delete_name` is not set, does nothing.
    """

    post_url_on_delete_name = "subject_dashboard_url"  # lookup key for url_names dict

    def __init__(self, *args):
        self.post_url_on_delete = None
        super().__init__(*args)

    def get_post_url_on_delete(self, request, obj):
        try:
            url_name = url_names.get(self.post_url_on_delete_name)
        except InvalidUrlName:
            if self.post_url_on_delete_name:
                raise
            url_name = None
        if url_name:
            kwargs = self.post_url_on_delete_kwargs(request, obj)
            post_url_on_delete = reverse(url_name, kwargs=kwargs)
            return post_url_on_delete
        return None

    def post_url_on_delete_kwargs(self, request, obj):
        """Returns kwargs needed to reverse the url.

        Override.
        """
        return {}

    def delete_model(self, request, obj):
        """Overridden to intercept the obj to reverse
        the post_url_on_delete
        """
        self.post_url_on_delete = self.get_post_url_on_delete(request, obj)
        obj.delete()

    def response_delete(self, request, obj_display, obj_id):
        """Overridden to redirect to `post_url_on_delete`, if not None."""
        if self.post_url_on_delete:
            opts = self.model._meta
            msg = 'The %(name)s "%(obj)s" was deleted successfully.' % {
                "name": force_str(opts.verbose_name),
                "obj": force_str(obj_display),
            }
            messages.add_message(request, messages.SUCCESS, msg)
            return HttpResponseRedirect(self.post_url_on_delete)
        return super().response_delete(request, obj_display, obj_id)



from django.apps import apps as django_apps
from django.utils.safestring import mark_safe
from simple_history.admin import SimpleHistoryAdmin as BaseSimpleHistoryAdmin


class SimpleHistoryAdmin(BaseSimpleHistoryAdmin):

    history_list_display = ["dashboard", "change_message"]
    object_history_template = "edc_model_admin/admin/object_history.html"
    object_history_form_template = "edc_model_admin/admin/object_history_form.html"

    save_as = False
    save_as_continue = False

    def change_message(self, obj):
        log_entry_model_cls = django_apps.get_model("admin.logentry")
        log_entry = (
            log_entry_model_cls.objects.filter(
                action_time__gte=obj.modified, object_id=str(obj.id)
            )
            .order_by("action_time")
            .first()
        )
        if log_entry:
            return mark_safe(log_entry.get_change_message())
        return None

    change_message.short_description = "Change Message"

    def dashboard(self, obj):
        if callable(self.view_on_site):
            return mark_safe(f'<A href="{self.view_on_site(obj)}">Dashboard</A>')
        return None



from django_revision.modeladmin_mixin import ModelAdminRevisionMixin
from edc_notification import NotificationModelAdminMixin

from edc_model_admin import (
    ModelAdminAuditFieldsMixin,
    ModelAdminFormAutoNumberMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminInstitutionMixin,
    ModelAdminNextUrlRedirectMixin,
    ModelAdminRedirectOnDeleteMixin,
    TemplatesModelAdminMixin,
)


class ModelAdminStackedInlineMixin(
    TemplatesModelAdminMixin,
    ModelAdminNextUrlRedirectMixin,
    NotificationModelAdminMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminFormAutoNumberMixin,
    ModelAdminRevisionMixin,
    ModelAdminAuditFieldsMixin,
    ModelAdminInstitutionMixin,
    ModelAdminRedirectOnDeleteMixin,
):
    pass



from django.contrib import admin

from .model_admin_audit_fields_mixin import audit_fieldset_tuple


class AddressModelAdminMixin(admin.ModelAdmin):

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "contact_name",
                    "name",
                    "address",
                    "postal_code",
                    "city",
                    "country",
                    "telephone",
                    "mobile",
                    "fax",
                    "email",
                )
            },
        ),
        audit_fieldset_tuple,
    )



import os

from django.conf import settings
from django.views.generic import ListView, TemplateView

from .models import CrfOne


class HomeView(TemplateView):
    template_name = os.path.join(
        settings.BASE_DIR, "edc_model_admin", "tests", "templates", "home.html"
    )


class CrfOneListView(ListView):
    model = CrfOne
    fields = ["subject_identifier", "subject_visit"]
    template_name = os.path.join(
        settings.BASE_DIR, "edc_model_admin", "tests", "templates", "crfmodel_list.html"
    )



from edc_dashboard.view_mixins import EdcViewMixin
from edc_dashboard.views import DashboardView as BaseDashboardView


class DashboardView(EdcViewMixin, BaseDashboardView):

    dashboard_url_name = "dashboard2_url"
    dashboard_template = "dashboard2_template"



from django.core.exceptions import ObjectDoesNotExist
from edc_fieldsets import FieldsetsModelAdminMixin
from edc_visit_tracking.modeladmin_mixins import CrfModelAdminMixin
from edc_visit_tracking.utils import get_subject_visit_model_cls

from .model_admin_subject_dashboard_mixin import ModelAdminSubjectDashboardMixin


class ModelAdminCrfDashboardMixin(
    FieldsetsModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    CrfModelAdminMixin,
):

    show_save_next = True
    show_cancel = True
    show_dashboard_in_list_display_pos = 1

    def get_subject_dashboard_url_kwargs(self, obj):
        return dict(
            subject_identifier=obj.subject_visit.subject_identifier,
            appointment=str(obj.subject_visit.appointment.id),
        )

    def get_changeform_initial_data(self, request):
        initial_data = super().get_changeform_initial_data(request)
        try:
            subject_visit = get_subject_visit_model_cls().objects.get(
                id=request.GET.get(self.model.visit_model_attr())
            )
        except ObjectDoesNotExist:
            # TODO: how do we get here?
            pass
        else:
            initial_data.update(
                report_datetime=subject_visit.report_datetime,
            )
        return initial_data



from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist
from django.test import tag
from django.urls.base import reverse
from django_webtest import WebTest
from edc_appointment.models import Appointment
from edc_constants.constants import YES
from edc_facility.import_holidays import import_holidays
from edc_lab.models.panel import Panel
from edc_lab.site_labs import site_labs
from edc_reference.site_reference import site_reference_configs
from edc_utils.date import get_utcnow
from edc_visit_schedule.site_visit_schedules import site_visit_schedules
from edc_visit_tracking.constants import SCHEDULED

from ..lab_profiles import lab_profile
from ..models import (
    CrfFive,
    CrfFour,
    CrfOne,
    CrfSix,
    CrfTwo,
    Requisition,
    SubjectConsent,
    SubjectVisit,
)
from ..reference_configs import register_to_site_reference_configs
from ..visit_schedule import visit_schedule

User = get_user_model()


class ModelAdminSiteTest(WebTest):
    @classmethod
    def setUpClass(cls):
        import_holidays()
        return super(ModelAdminSiteTest, cls).setUpClass()

    def setUp(self):
        self.user = User.objects.create_superuser("user_login", "u@example.com", "pass")

        site_labs._registry = {}
        site_labs.loaded = False
        site_labs.register(lab_profile=lab_profile)

        register_to_site_reference_configs()
        site_visit_schedules._registry = {}
        site_visit_schedules.loaded = False
        site_visit_schedules.register(visit_schedule)
        site_reference_configs.register_from_visit_schedule(
            visit_models={"edc_appointment.appointment": "edc_model_admin.subjectvisit"}
        )

        self.subject_identifier = "101-12345"
        SubjectConsent.objects.create(
            subject_identifier=self.subject_identifier,
            consent_datetime=get_utcnow(),
            identity="1111111",
            confirm_identity="1111111",
            visit_schedule_name="visit_schedule",
            schedule_name="schedule",
        )
        appointment = Appointment.objects.get(visit_code="1000")
        self.subject_visit = SubjectVisit.objects.create(
            appointment=appointment, report_datetime=get_utcnow(), reason=SCHEDULED
        )

    def login(self):
        form = self.app.get(reverse("admin:index")).maybe_follow().form
        form["username"] = self.user.username
        form["password"] = "pass"
        return form.submit()

    def test_redirect_next(self):
        """Assert redirects to "dashboard_url" as give in the
        query_string "next=".
        """
        self.login()

        self.app.get(
            reverse("dashboard_app:dashboard_url", args=(self.subject_identifier,)),
            user=self.user,
            status=200,
        )

        CrfOne.objects.create(subject_visit=self.subject_visit, report_datetime=get_utcnow())

        model = "redirectnextmodel"
        query_string = (
            "next=dashboard_app:dashboard_url,subject_identifier&"
            f"subject_identifier={self.subject_identifier}"
        )

        url = reverse(f"admin:edc_model_admin_{model}_add") + "?" + query_string

        response = self.app.get(url, user=self.user)
        response.form["subject_identifier"] = self.subject_identifier
        response = response.form.submit(name="_save").follow()

        self.assertIn("You are at the subject dashboard", response)
        self.assertIn(self.subject_identifier, response)

    @tag("1")
    def test_redirect_save_next_crf(self):
        """Assert redirects CRFs for both add and change from
        crftwo -> crfthree -> dashboard.
        """
        self.login()

        self.app.get(
            reverse("dashboard_app:dashboard_url", args=(self.subject_identifier,)),
            user=self.user,
            status=200,
        )

        # add CRF Two
        model = "crftwo"
        query_string = (
            "next=dashboard_app:dashboard_url,subject_identifier&"
            f"subject_identifier={self.subject_identifier}"
        )
        url = reverse(f"admin:edc_model_admin_{model}_add") + "?" + query_string

        # oops, cancel
        response = self.app.get(url, user=self.user)
        response = response.form.submit(name="_cancel").follow()
        self.assertIn("You are at the subject dashboard", response)

        # add CRF Two
        response = self.app.get(url, user=self.user)
        self.assertIn("Add crf two", response)
        form_data = {
            "subject_visit": str(self.subject_visit.id),
            "report_datetime_0": get_utcnow().strftime("%Y-%m-%d"),
            "report_datetime_1": "00:00:00",
        }
        for key, value in form_data.items():
            response.form[key] = value
        response = response.form.submit(name="_savenext").follow()

        # goes directly to CRF Three, add CRF Three
        self.assertIn("Add crf three", response)
        form_data = {
            "subject_visit": str(self.subject_visit.id),
            "report_datetime_0": get_utcnow().strftime("%Y-%m-%d"),
            "report_datetime_1": "00:00:00",
        }
        for key, value in form_data.items():
            response.form[key] = value
        response = response.form.submit(name="_savenext").follow()

        # goes to dashboard
        self.assertIn("You are at the subject dashboard", response)
        self.assertIn(self.subject_identifier, response)

        crftwo = CrfTwo.objects.all()[0]

        url = (
            reverse(f"admin:edc_model_admin_{model}_change", args=(crftwo.id,))
            + "?"
            + query_string
        )

        response = self.app.get(url, user=self.user)
        response = response.form.submit(name="_cancel").follow()
        self.assertIn("You are at the subject dashboard", response)

        response = self.app.get(url, user=self.user)
        self.assertIn("Change crf two", response)
        form_data = {
            "subject_visit": str(self.subject_visit.id),
            "report_datetime_0": get_utcnow().strftime("%Y-%m-%d"),
            "report_datetime_1": "00:00:00",
        }
        for key, value in form_data.items():
            response.form[key] = value
        response = response.form.submit(name="_savenext").follow()

        self.assertIn("You are at the subject dashboard", response)
        self.assertIn(self.subject_identifier, response)

    def test_redirect_save_next_requisition(self):
        """Assert redirects requisitions for both add and change from
        panel one -> panel two -> dashboard.
        """
        self.login()

        self.app.get(
            reverse("dashboard_app:dashboard_url", args=(self.subject_identifier,)),
            user=self.user,
            status=200,
        )

        model = "requisition"
        query_string = (
            "next=dashboard_app:dashboard_url,subject_identifier&"
            f"subject_identifier={self.subject_identifier}&"
            f"subject_visit={str(self.subject_visit.id)}"
        )

        panel_one = Panel.objects.get(name="one")
        panel_two = Panel.objects.get(name="two")

        # got to add and cancel
        add_url = reverse(f"admin:edc_model_admin_{model}_add")
        url = add_url + f"?{query_string}&panel={str(panel_one.id)}"
        response = self.app.get(url, user=self.user)
        response = response.form.submit(name="_cancel").follow()
        self.assertIn("You are at the subject dashboard", response)

        dte = get_utcnow()
        form_data = {
            "item_count": 1,
            "estimated_volume": 5,
            "is_drawn": YES,
            "drawn_datetime_0": dte.strftime("%Y-%m-%d"),
            "drawn_datetime_1": "00:00:00",
            "clinic_verified": YES,
            "clinic_verified_datetime_0": dte.strftime("%Y-%m-%d"),
            "clinic_verified_datetime_1": "00:00:00",
        }

        # add and save
        url = add_url + f"?{query_string}&panel={str(panel_one.id)}"
        response = self.app.get(url, user=self.user)
        self.assertIn("Add requisition", response)
        self.assertIn(f'value="{str(panel_one.id)}"', response)
        for key, value in form_data.items():
            response.form[key] = value
        response.form["requisition_identifier"] = "ABCDE0001"
        response = response.form.submit().follow()
        self.assertIn("You are at the subject dashboard", response)
        Requisition.objects.all().delete()

        # add panel one and save_next ->
        # add panel two and save_next -> dashboard
        url = add_url + f"?{query_string}&panel={str(panel_one.id)}"
        response = self.app.get(url, user=self.user)
        self.assertIn("Add requisition", response)
        self.assertIn(f'value="{str(panel_one.id)}"', response)
        self.assertIn("_savenext", response)
        for key, value in form_data.items():
            response.form[key] = value
        response.form["requisition_identifier"] = "ABCDE0001"
        response = response.form.submit(name="_savenext").follow()
        self.assertIn("Add requisition", response)
        self.assertIn(f'value="{str(panel_two.id)}"', response)
        for key, value in form_data.items():
            response.form[key] = value
        response.form["requisition_identifier"] = "ABCDE0002"
        response = response.form.submit(name="_savenext").follow()
        self.assertIn("You are at the subject dashboard", response)
        self.assertIn(self.subject_identifier, response)

        # change panel one and save_next -> change panel two and save_next ->
        # dashboard
        requisition = Requisition.objects.get(requisition_identifier="ABCDE0001")
        url = (
            reverse(f"admin:edc_model_admin_{model}_change", args=(requisition.id,))
            + f"?{query_string}&panel={str(panel_one.id)}"
        )
        response = self.app.get(url, user=self.user)
        self.assertIn("Change requisition", response)
        self.assertIn("ABCDE0001", response)
        self.assertIn(f'{str(panel_one.id)}" selected>One</option>', response)
        response = response.form.submit(name="_savenext").follow()

        # self.assertIn("Change requisition", response)
        # self.assertIn("ABCDE0002", response)
        # self.assertIn(f'{str(panel_two.id)}" selected>Two</option>', response)
        # self.assertIn(str(panel_two.id), response)
        # response = response.form.submit(name="_savenext").follow()

        self.assertIn("You are at the subject dashboard", response)
        self.assertIn(self.subject_identifier, response)

    def test_redirect_on_delete_with_url_name_from_settings(self):

        self.login()

        self.app.get(
            reverse("dashboard_app:dashboard_url", args=(self.subject_identifier,)),
            user=self.user,
            status=200,
        )

        model = "crffour"
        query_string = (
            "next=dashboard_app:dashboard_url,subject_identifier&"
            f"subject_identifier={self.subject_identifier}"
        )
        url = reverse(f"admin:edc_model_admin_{model}_add") + "?" + query_string

        form_data = {
            "subject_visit": str(self.subject_visit.id),
            "report_datetime_0": get_utcnow().strftime("%Y-%m-%d"),
            "report_datetime_1": "00:00:00",
        }
        response = self.app.get(url, user=self.user)
        for key, value in form_data.items():
            response.form[key] = value
        response.form.submit(name="_save").follow()

        # delete
        crffour = CrfFour.objects.all()[0]
        url = (
            reverse(f"admin:edc_model_admin_{model}_change", args=(crffour.id,))
            + "?"
            + query_string
        )
        response = self.app.get(url, user=self.user)
        delete_url = reverse(f"admin:edc_model_admin_{model}_delete", args=(crffour.id,))
        response = response.click(href=delete_url)

        # submit confirmation page
        response = response.form.submit().follow()

        # redirects to the dashboard
        self.assertIn("You are at the subject dashboard", response)
        self.assertRaises(ObjectDoesNotExist, CrfFour.objects.get, id=crffour.id)

    def test_redirect_on_delete_with_url_name_from_admin(self):
        self.login()

        crffive = CrfFive.objects.create(
            subject_visit=self.subject_visit, report_datetime=get_utcnow()
        )

        model = "crffive"
        url = reverse(f"admin:edc_model_admin_{model}_change", args=(crffive.id,))
        response = self.app.get(url, user=self.user)
        delete_url = reverse(f"admin:edc_model_admin_{model}_delete", args=(crffive.id,))
        response = response.click(href=delete_url)
        response = response.form.submit().follow()
        self.assertIn("You are at Dashboard Two", response)
        self.assertRaises(ObjectDoesNotExist, CrfFive.objects.get, id=crffive.id)

    def test_redirect_on_delete_with_url_name_is_none(self):
        self.login()

        crfsix = CrfSix.objects.create(
            subject_visit=self.subject_visit, report_datetime=get_utcnow()
        )

        model = "crfsix"
        url = reverse(f"admin:edc_model_admin_{model}_change", args=(crfsix.id,))
        response = self.app.get(url, user=self.user)
        delete_url = reverse(f"admin:edc_model_admin_{model}_delete", args=(crfsix.id,))
        response = response.click(href=delete_url)
        response = response.form.submit().follow()
        self.assertRaises(ObjectDoesNotExist, CrfSix.objects.get, id=crfsix.id)
        self.assertIn("changelist", response)



from django.contrib import admin
from django.urls import include, path
from django.views.generic.base import RedirectView

from .admin import *  # noqa
from .views import CrfOneListView

app_name = "edc_model_admin"

urlpatterns = []

for app_name in [
    "edc_dashboard",
    "edc_consent",
    "edc_device",
    "edc_protocol",
    "edc_reference",
    "edc_visit_schedule",
]:
    urlpatterns.append(path(f"{app_name}/", include(f"{app_name}.urls")))

urlpatterns += [
    path("admin/", admin.site.urls),
    path("", CrfOneListView.as_view(), name="crfone-list"),
    path("", include("edc_model_admin.tests.dashboard_app.urls")),
    path("", include("edc_model_admin.tests.dashboard2_app.urls")),
    path("", RedirectView.as_view(url="admin/"), name="administration_url"),
    path("", RedirectView.as_view(url="admin/"), name="home_url"),
    path("", RedirectView.as_view(url="admin/"), name="logout"),
]



from .address_model_admin_mixin import AddressModelAdminMixin
from .changelist_buttons import (
    ModelAdminChangelistButtonMixin,
    ModelAdminChangelistModelButtonMixin,
)
from .inlines import (
    LimitedAdminInlineMixin,
    StackedInlineModelAdminMixin,
    TabularInlineMixin,
)
from .model_admin_audit_fields_mixin import (
    ModelAdminAuditFieldsMixin,
    audit_fields,
    audit_fieldset_tuple,
)
from .model_admin_form_auto_number_mixin import ModelAdminFormAutoNumberMixin
from .model_admin_form_instructions_mixin import ModelAdminFormInstructionsMixin
from .model_admin_institution_mixin import ModelAdminInstitutionMixin
from .model_admin_model_redirect_mixin import ModelAdminModelRedirectMixin
from .model_admin_next_url_redirect_mixin import (
    ModelAdminNextUrlRedirectError,
    ModelAdminNextUrlRedirectMixin,
)
from .model_admin_redirect_on_delete_mixin import ModelAdminRedirectOnDeleteMixin
from .model_admin_replace_label_text_mixin import ModelAdminReplaceLabelTextMixin
from .model_admin_simple_history import SimpleHistoryAdmin
from .templates_model_admin_mixin import TemplatesModelAdminMixin
from .utils import get_next_url



from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.urls import NoReverseMatch, reverse
from django.utils.translation import gettext as _
from django_revision.modeladmin_mixin import ModelAdminRevisionMixin
from edc_dashboard.url_names import url_names
from edc_notification import NotificationModelAdminMixin
from edc_registration.models import RegisteredSubject

from edc_model_admin import (
    ModelAdminAuditFieldsMixin,
    ModelAdminFormAutoNumberMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminInstitutionMixin,
    ModelAdminNextUrlRedirectMixin,
    ModelAdminRedirectOnDeleteMixin,
    ModelAdminReplaceLabelTextMixin,
    TemplatesModelAdminMixin,
)


class ModelAdminSubjectDashboardMixin(
    TemplatesModelAdminMixin,
    ModelAdminNextUrlRedirectMixin,
    NotificationModelAdminMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminFormAutoNumberMixin,
    ModelAdminRevisionMixin,
    ModelAdminAuditFieldsMixin,
    ModelAdminInstitutionMixin,
    ModelAdminRedirectOnDeleteMixin,
    ModelAdminReplaceLabelTextMixin,
):

    date_hierarchy = "modified"
    empty_value_display = "-"
    list_per_page = 10
    subject_dashboard_url_name = "subject_dashboard_url"
    subject_listboard_url_name = "subject_listboard_url"
    show_cancel = True
    show_dashboard_in_list_display_pos = None

    def get_subject_dashboard_url_name(self):
        return url_names.get(self.subject_dashboard_url_name)

    def get_subject_dashboard_url_kwargs(self, obj):
        return dict(subject_identifier=obj.subject_identifier)

    def get_subject_listboard_url_name(self):
        return url_names.get(self.subject_listboard_url_name)

    def get_post_url_on_delete_name(self, *args):
        return self.get_subject_dashboard_url_name()

    def post_url_on_delete_kwargs(self, request, obj):
        return self.get_subject_dashboard_url_kwargs(obj)

    def dashboard(self, obj=None, label=None):
        url = reverse(
            self.get_subject_dashboard_url_name(),
            kwargs=self.get_subject_dashboard_url_kwargs(obj),
        )
        context = dict(title=_("Go to subject's dashboard"), url=url, label=label)
        return render_to_string("dashboard_button.html", context=context)

    def get_list_display(self, request):
        list_display = super().get_list_display(request)
        if self.show_dashboard_in_list_display_pos is not None:
            list_display = list(list_display)
            if self.dashboard not in list_display:
                list_display.insert(self.show_dashboard_in_list_display_pos, self.dashboard)
        return list_display

    def view_on_site(self, obj):
        try:
            RegisteredSubject.objects.get(subject_identifier=obj.subject_identifier)
        except ObjectDoesNotExist:
            url = reverse(self.get_subject_listboard_url_name())
        else:
            try:
                url = reverse(
                    self.get_subject_dashboard_url_name(),
                    kwargs=self.get_subject_dashboard_url_kwargs(obj),
                )
            except NoReverseMatch as e:
                if callable(super().view_on_site):
                    url = super().view_on_site(obj)
                else:
                    raise NoReverseMatch(
                        f"{e}. See subject_dashboard_url_name for {repr(self)}."
                    )
        return url



from django.http.response import HttpResponseRedirect


class BaseModelAdminRedirectMixin:

    """Redirect on add, change, or delete."""

    def redirect_url(self, request, obj, post_url_continue=None):
        return None

    def redirect_url_on_add(self, request, obj, post_url_continue=None):
        return self.redirect_url(request, obj, post_url_continue=post_url_continue)

    def redirect_url_on_change(self, request, obj, post_url_continue=None):
        return self.redirect_url(request, obj, post_url_continue=post_url_continue)

    def redirect_url_on_delete(self, request, obj_display, obj_id):
        return None

    def response_add(self, request, obj, post_url_continue=None):
        redirect_url = None
        if "_addanother" not in request.POST and "_continue" not in request.POST:
            redirect_url = self.redirect_url_on_add(request, obj)
        if redirect_url:
            return HttpResponseRedirect(redirect_url)
        return super().response_add(request, obj, post_url_continue=post_url_continue)

    def response_change(self, request, obj):
        redirect_url = None
        if "_addanother" not in request.POST and "_continue" not in request.POST:
            redirect_url = self.redirect_url_on_change(request, obj)
        if redirect_url:
            return HttpResponseRedirect(redirect_url)
        return super().response_change(request, obj)

    def response_delete(self, request, obj_display, obj_id):
        redirect_url = self.redirect_url_on_delete(request, obj_display, obj_id)
        if redirect_url:
            return HttpResponseRedirect(redirect_url)
        return super().response_delete(request, obj_display, obj_id)



from warnings import warn

from django.urls.base import reverse
from django.urls.exceptions import NoReverseMatch


def get_next_url(request, next_attr=None, warn_to_console=None):

    url = None
    next_value = request.GET.dict().get(next_attr or "next")
    warn_to_console = True if warn_to_console is None else warn_to_console

    if next_value:
        kwargs = {}
        for pos, value in enumerate(next_value.split(",")):
            if pos == 0:
                next_url = value
            else:
                kwargs.update({value: request.GET.get(value)})
        try:
            url = reverse(next_url, kwargs=kwargs)
        except NoReverseMatch as e:
            if warn_to_console:
                warn(f"{e}. Got {next_value}.")
    return url



from warnings import warn

from django import template
from django.contrib.admin.templatetags.admin_modify import (
    submit_row as django_submit_row,
)
from django.urls.base import reverse
from django.urls.exceptions import NoReverseMatch

from edc_model_admin import get_next_url

register = template.Library()


class EdcContextProcessorError(Exception):
    pass


def get_request_object(context):
    """Returns a request object or raises EdcContextProcessorError."""
    request = context.get("request")
    if not request:
        raise EdcContextProcessorError(
            "Request object not found in template context. "
            "Try enabling the context processor "
            "'django.template.context_processors.request'"
        )
    return request


def get_subject_identifier(context):
    """Returns the subject identifier."""
    request = get_request_object(context)
    subject_identifier = request.GET.get("subject_identifier")
    if not subject_identifier:
        try:
            subject_identifier = context["subject_identifier"]
        except KeyError:
            try:
                subject_identifier = context["original"].subject_identifier
            except (KeyError, AttributeError):
                subject_identifier = None
    return subject_identifier


def get_cancel_url(context, cancel_attr=None):
    """Returns the url for the Cancel button on the change_form."""
    request = get_request_object(context)
    cancel_url = request.GET.dict().get("cancel_url")
    if not cancel_url:
        cancel_querystring = request.GET.dict().get(cancel_attr or "cancel")
        if cancel_querystring:
            url = None
            kwargs = {}
            for pos, value in enumerate(cancel_querystring.split(",")):
                if pos == 0:
                    url = value
                else:
                    kwargs.update({value: request.GET.get(value)})
            try:
                cancel_url = reverse(url, kwargs=kwargs)
            except NoReverseMatch as e:
                warn(f"{e}. Got {cancel_url}.")
        else:
            cancel_url = get_next_url(request, warn_to_console=False)
            if not cancel_url:
                url = context["subject_dashboard_url"]
                kwargs = {"subject_identifier": get_subject_identifier(context)}
                try:
                    cancel_url = reverse(url, kwargs=kwargs)
                except NoReverseMatch as e:
                    cancel_url = None
                    warn(f"{str(e)} kwargs={kwargs}. See 'get_cancel_url'.")
    return cancel_url


@register.inclusion_tag("edc_submit_line.html", takes_context=True)
def edc_submit_row(context):

    request = get_request_object(context)
    if request:
        if int(request.site.id) == int(context.get("reviewer_site_id", 0)):
            context.update({"save_next": None})
            context.update({"show_delete": None})

    show_save = context.get("show_save")
    if "save_next" in context:
        context["save_next"] = show_save

    if "show_cancel" in context:
        context["cancel_url"] = get_cancel_url(context)

    return django_submit_row(context)


@register.inclusion_tag("edc_revision_line.html", takes_context=True)
def revision_row(context):
    return dict(
        copyright=context.get("copyright"),
        institution=context.get("institution"),
        revision=context.get("revision"),
        disclaimer=context.get("disclaimer"),
    )


@register.inclusion_tag("edc_instructions.html", takes_context=True)
def instructions(context):
    instructions = context.get("instructions")
    return {"instructions": instructions}


@register.inclusion_tag("edc_additional_instructions.html", takes_context=True)
def additional_instructions(context):
    additional_instructions = context.get("additional_instructions")
    notification_instructions = context.get("notification_instructions")
    return {
        "additional_instructions": additional_instructions,
        "notification_instructions": notification_instructions,
    }



from dateutil.relativedelta import relativedelta
from edc_visit_schedule import (
    Crf,
    FormsCollection,
    Requisition,
    Schedule,
    Visit,
    VisitSchedule,
)
from edc_visit_schedule.tests import DummyPanel

from .lab_profiles import panel_one, panel_two

app_label = "edc_model_admin"


class MockPanel(DummyPanel):
    """`requisition_model` is normally set when the lab profile
    is set up.
    """

    def __init__(self, name):
        super().__init__(requisition_model=f"{app_label}.requisition", name=name)


crfs0 = FormsCollection(
    Crf(show_order=1, model=f"{app_label}.crfone", required=True),
    Crf(show_order=2, model=f"{app_label}.crftwo", required=True),
    Crf(show_order=3, model=f"{app_label}.crfthree", required=True),
    # Crf(show_order=4, model=f"{app_label}.crffour", required=True),
    # Crf(show_order=5, model=f"{app_label}.crffive", required=True),
)

crfs1 = FormsCollection(
    Crf(show_order=1, model=f"{app_label}.crffour", required=True),
    Crf(show_order=2, model=f"{app_label}.crffive", required=True),
    Crf(show_order=3, model=f"{app_label}.crfsix", required=True),
)

crfs2 = FormsCollection(Crf(show_order=1, model=f"{app_label}.crfseven", required=True))


crfs_unscheduled = FormsCollection(
    Crf(show_order=1, model=f"{app_label}.crftwo", required=True),
    Crf(show_order=2, model=f"{app_label}.crfthree", required=True),
    Crf(show_order=3, model=f"{app_label}.crffive", required=True),
)

requisitions = FormsCollection(
    Requisition(show_order=10, panel=panel_one, required=True, additional=False),
    Requisition(show_order=20, panel=panel_two, required=True, additional=False),
)

requisitions3000 = FormsCollection(
    Requisition(show_order=10, panel=MockPanel("seven"), required=True, additional=False)
)

requisitions_unscheduled = FormsCollection(
    Requisition(show_order=10, panel=MockPanel("one"), required=True, additional=False),
    Requisition(show_order=20, panel=MockPanel("three"), required=True, additional=False),
    Requisition(show_order=30, panel=MockPanel("five"), required=True, additional=False),
)

visit0 = Visit(
    code="1000",
    title="Day 1",
    timepoint=0,
    rbase=relativedelta(days=0),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=6),
    requisitions=requisitions,
    crfs=crfs0,
    crfs_unscheduled=crfs_unscheduled,
    requisitions_unscheduled=requisitions_unscheduled,
    allow_unscheduled=True,
    facility_name="5-day-clinic",
)

visit1 = Visit(
    code="2000",
    title="Day 2",
    timepoint=1,
    rbase=relativedelta(days=1),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=6),
    requisitions=requisitions,
    crfs=crfs1,
    facility_name="5-day-clinic",
)

visit2 = Visit(
    code="3000",
    title="Day 3",
    timepoint=2,
    rbase=relativedelta(days=2),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=6),
    requisitions=requisitions3000,
    crfs=crfs2,
    facility_name="5-day-clinic",
)

schedule = Schedule(
    name="schedule",
    onschedule_model=f"{app_label}.onschedule",
    offschedule_model=f"{app_label}.offschedule",
    consent_model=f"{app_label}.subjectconsent",
    appointment_model="edc_appointment.appointment",
)

schedule.add_visit(visit0)
schedule.add_visit(visit1)
schedule.add_visit(visit2)

visit_schedule = VisitSchedule(
    name="visit_schedule",
    offstudy_model="edc_offstudy.subjectoffstudy",
    death_report_model=f"{app_label}.deathreport",
)

visit_schedule.add_schedule(schedule)



from django.contrib import admin

from edc_model_admin import (
    ModelAdminNextUrlRedirectMixin,
    ModelAdminRedirectOnDeleteMixin,
    TemplatesModelAdminMixin,
)

from .models import (
    CrfFive,
    CrfFour,
    CrfOne,
    CrfSix,
    CrfThree,
    CrfTwo,
    RedirectNextModel,
    Requisition,
)

__all__ = [
    "CrfOneAdmin",
    "RedirectNextModelAdmin",
    "CrfTwoAdmin",
    "CrfThreeAdmin",
    "RequisitionAdmin",
    "CrfFourAdmin",
    "CrfFiveAdmin",
    "CrfSixAdmin",
]


class BaseModelAdmin(TemplatesModelAdminMixin):

    search_fields = ("subject_identifier",)


@admin.register(CrfOne)
class CrfOneAdmin(BaseModelAdmin, admin.ModelAdmin):
    pass


# using ModelAdminNextUrlRedirectMixin


@admin.register(RedirectNextModel)
class RedirectNextModelAdmin(BaseModelAdmin, ModelAdminNextUrlRedirectMixin, admin.ModelAdmin):
    pass


@admin.register(CrfTwo)
class CrfTwoAdmin(BaseModelAdmin, ModelAdminNextUrlRedirectMixin, admin.ModelAdmin):
    show_save_next = True
    show_cancel = True


@admin.register(CrfThree)
class CrfThreeAdmin(BaseModelAdmin, ModelAdminNextUrlRedirectMixin, admin.ModelAdmin):
    pass


@admin.register(Requisition)
class RequisitionAdmin(BaseModelAdmin, ModelAdminNextUrlRedirectMixin, admin.ModelAdmin):
    show_save_next = True
    show_cancel = True


# using ModelAdminRedirectOnDeleteMixin


@admin.register(CrfFour)
class CrfFourAdmin(BaseModelAdmin, ModelAdminRedirectOnDeleteMixin, admin.ModelAdmin):

    post_url_on_delete_name = "dashboard_url"

    def post_url_on_delete_kwargs(self, request, obj):
        return {"subject_identifier": obj.subject_identifier}


@admin.register(CrfFive)
class CrfFiveAdmin(BaseModelAdmin, ModelAdminRedirectOnDeleteMixin, admin.ModelAdmin):

    post_url_on_delete_name = "dashboard2_url"

    def post_url_on_delete_kwargs(self, request, obj):
        return {"subject_identifier": obj.subject_identifier}


@admin.register(CrfSix)
class CrfSixAdmin(BaseModelAdmin, ModelAdminRedirectOnDeleteMixin, admin.ModelAdmin):

    post_url_on_delete_name = None

    def post_url_on_delete_kwargs(self, request, obj):
        return {"subject_identifier": obj.subject_identifier}



from django.utils.safestring import mark_safe

NAME = 0
WIDGET = 1


class ModelAdminReplaceLabelTextMixin:
    @staticmethod
    def replace_label_text(form=None, old=None, new=None, skip_fields=None):
        skip_fields = skip_fields or []
        for fld in form.base_fields.items():
            if fld[NAME] not in skip_fields:
                label = str(fld[WIDGET].label)
                if old in label:
                    label = label.replace(old, new)
                    fld[WIDGET].label = mark_safe(label)
        return form



from django.urls import reverse
from django.utils.html import format_html


class ModelAdminChangelistButtonMixin:

    changelist_model_button_template = (
        '<a href="{{url}}" class="button" title="{{title}}" {{disabled}}>{label}</a>'
    )

    def button(
        self,
        url_name,
        reverse_args,
        disabled=None,
        label=None,
        title=None,
        namespace=None,
    ):
        label = label or "change"
        if namespace:
            url_name = f"{namespace}:{url_name}"
        url = reverse(url_name, args=reverse_args)
        return self.button_template(label, url=url, disabled=disabled, title=title)

    def change_button(
        self,
        url_name,
        reverse_args,
        disabled=None,
        label=None,
        title=None,
        namespace=None,
    ):
        label = label or "change"
        if namespace:
            url_name = f"{namespace}:{url_name}"
        url = reverse(url_name, args=reverse_args)
        return self.button_template(label, url=url, disabled=disabled, title=title)

    def add_button(
        self,
        url_name,
        disabled=None,
        label=None,
        querystring=None,
        namespace=None,
        title=None,
    ):
        label = label or "add"
        if namespace:
            url_name = f"{namespace}:{url_name}"
        url = reverse(url_name) + "" if querystring is None else querystring
        return self.button_template(label, url=url, disabled=disabled, title=title)

    def button_template(self, label, disabled=None, title=None, url=None):
        title = title or ""
        disabled = "disabled" if disabled else ""
        if disabled or not url:
            url = "#"
        button_template = self.changelist_model_button_template.format(label=label)
        button_template = format_html(button_template, disabled=disabled, title=title, url=url)
        return button_template



from edc_lab import AliquotType, LabProfile, ProcessingProfile, RequisitionPanel

wb = AliquotType(name="Whole Blood", alpha_code="WB", numeric_code="02")


fbc_processing = ProcessingProfile(name="FBC", aliquot_type=wb)

panel_one = RequisitionPanel(name="one", verbose_name="One", processing_profile=fbc_processing)

panel_two = RequisitionPanel(name="two", verbose_name="Two", processing_profile=fbc_processing)

lab_profile = LabProfile(name="lab_profile", requisition_model="edc_model_admin.requisition")

lab_profile.add_panel(panel_one)
lab_profile.add_panel(panel_two)
